CON

'Resulting clock frequency must be 80 MHz:

   _clkmode       = xtal1 + pll16x
   _xinfreq       = 5_000_000

'ASCII quote and newline characters:

   QU             = $22
   LF             = $0a

'Pin assignments for Spinneret. Can be changed to accommodate other systems:

   SCL            = 28
   SDA            = 29
   VSYNC          = 30
   MCLK           = 31
   SERDAT         = 24
   HSYNC          = 27

DAT

my_ip         byte      "192.168.0.50:3456", 0  'Change to desired IP address and port number for your Spinneret.
my_gateway    byte      "192.168.0.4", 0        'Change to IP address of your internet gateway.
my_subnetmask byte      "255.255.255.0", 0      'Probably do not need to change this.
my_macaddr    byte      "00 08 DC 16 F0 13", 0  'Change to value printed on Spinneret label.

my_pins       byte      SCL, SDA, VSYNC, MCLK, SERDAT, HSYNC     
  
OBJ

  srv   : "http_helper"
  bmp   : "Bitmap_header"
  cam   : "PropCAM_serial6"  
  
PUB start | request

  gray_buf_addr := @gray_buf
  sync_addr := @sync
  cam.start(cam#CONTINUOUS, @my_pins, @gray_buf_addr)
  
  srv.start(@my_ip, @my_gateway, @my_subnetmask, @my_macaddr)
  dec(cam#BITS_PER_PIXEL, @pix_, 1)
  

  repeat
    request := srv.request
    if (strcomp(request, string("/index")) or strcomp(request, string("/")))
      main_page
    elseif (strcomp(request, string("/webcam.bmp")))
      webcam_image
    else
      srv.not_found

PUB main_page

  dec(gain, @gain_, 2)
  dec(exp_time, @expos_, 4)
  dec(avg_pix, @avg_, 4)
  srv.begin_html_page(string("PropCAM Webcam Demo"), string("2"), string("#ccccaa"))
  srv.str(@html)
  srv.end_html_page
      
PUB webcam_image | i, j, pix_long, pix

  cam.acquire(cam#GRAB)
  srv.array(bmp.header(cam#PIXELS_PER_ROW, cam#ROWS_PER_FRAME, cam#BITS_PER_PIXEL, 0), bmp.header_size)
  repeat i from 0 to constant(cam#PIXEL_DEPTH - 1)
    pix_long := i * constant(256 / cam#PIXEL_DEPTH)
    srv.rep(pix_long, 3)
  if (cam#BITS_PER_PIXEL > 4)
    srv.rep(0, constant((256 - cam#PIXEL_DEPTH) * 3))
  elseif (cam#BITS_PER_PIXEL > 1)
    srv.rep(0, constant((16 - cam#PIXEL_DEPTH) * 3))
  repeat i from constant(cam#ROWS_PER_FRAME - 1) to 0
    repeat j from 0 to constant(cam#LONGS_PER_ROW - 1)
      pix_long := long[@gray_buf][i * cam#LONGS_PER_ROW + j + 1]
      if (cam#PIXELS_PER_BYTE == 2)
        pix_long := (pix_long & $0f0f_0f0f) << 4 | (pix_long & $f0f0_f0f0) >> 4
      repeat constant(cam#PIXELS_PER_LONG / cam#PIXELS_PER_BYTE)
        pix := pix_long & cam#BYTE_MASK
        if (cam#PIXELS_PER_BYTE == 8)
          pix ><= 8
        srv.out(pix)
        pix_long >>= cam#BYTE_SHIFT
    srv.rep(0, constant((4 - cam#PIXELS_PER_ROW // 4)) & 3)
  srv.end_custom_content

PUB dec(value, addr, width) | digit

  digit~
  repeat while ((value or digit == 0) and width)
    digit := value // 10 + "0"
    byte[addr + --width]  := value // 10 + "0"
    value /= 10
  repeat while (width--)
    byte[addr + width] := " "  
                     
DAT
              word      @gain_ - @html

html          byte      "    <div style=", QU, "font-family:sans-serif;font-weight:bold;font-size:large;color:navy;text-align:center", QU, ">", LF
              byte      "      PropCAM "
pix_          byte      "0-bit Serial Webcam Demo<p>", LF
              byte      "      <img src=", QU, "/webcam.bmp", QU, " width=375 height=288 /><p>", LF
              byte      "      Gain: "
gain_         byte      "00 &nbsp; &nbsp; Exposure: "
expos_        byte      "0000 &nbsp; &nbsp; Avg Pix: "
avg_          byte      "0000", LF
              byte      "    </div>", 0

gray_buf      long      cam#PIXELS_PER_ROW << 16 | cam#ROWS_PER_FRAME, 0[cam#LONGS_PER_ROW * cam#ROWS_PER_FRAME]

 
gray_buf_addr word      0-0
left_pix      word      0
top_pix       word      0
exp_type      word      cam#AUTOBOTH
sync_addr     word      0-0
exp_time      word      2048
gain          word      15
target        word      800
time_interval word      -30
avg_pix       word      0-0

sync          byte      0
